using System;
using System.Collections.Generic;
using System.Runtime.ExceptionServices;
using System.Xml.Serialization;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;


namespace Matopeli;

/// @author Omanimi
/// @version 26.10.2023
/// <summary>
/// 
/// </summary>
    public class Matopeli : PhysicsGame
    {

        private static readonly string[] kentta ={
            "---------------",
            "-             -",
            "-             -",
            "-             -",
            "-             -",
            "-             -",
            "-    m        -",
            "-             -",
            "-             -",
            "-             -",
            "-             -",
            "-             -",
            "-             -",
            "---------------",
        };
        private int seinaleveys= 600 / kentta.Length;
        private int seinakorkeus = 600 / kentta.Length;

        private static readonly Image seinakuva = LoadImage("seina.png");
        private static readonly Image matopaakuva = LoadImage("matopaa.png");
        private readonly Image matokehokuva = LoadImage("matokeho.png");
        private readonly Image omenakuva = LoadImage("omena.png");
        private readonly Image kenttatausta = LoadImage("kenttatausta.png");

        private LiikkuvaMato mato;
        private LiikkuvaMato omena;
        
        public readonly IntMeter pisteet = new IntMeter(0);


        public override void Begin()
        {
            ClearAll();
           // Background.Image = LoadImage("kenttatausta");
            //LuoTausta();
        
            AloitaUusiPeli();
        }

        public void AloitaUusiPeli()
        {
            // pisteet.Value = 0;

            TileMap seinat = TileMap.FromStringArray(kentta);
            seinat.SetTileMethod('-', LuoSeina);
            seinat['m'] = LuoMato;
            //seinat.SetTileMethod('k',LuoMatokeho);
            seinat.Insert(seinaleveys, seinakorkeus);
            Asetaohjaimet();

            seinat.Execute(seinaleveys, seinakorkeus);
            LuoOmena();
        }



        private void Asetaohjaimet()
        {
            Keyboard.Listen(Key.Up, ButtonState.Down, KaannyYlos, "Turn upwards");
            Keyboard.Listen(Key.Down, ButtonState.Down, KaannyAlas, "Turn downwards");
            Keyboard.Listen(Key.Left, ButtonState.Down, KaannyVasen, "Turn left");
            Keyboard.Listen(Key.Right, ButtonState.Down, KaannyOikea, "Turn right");
        }

        private void KaannyYlos()
        {
            
            mato.Velocity = new Vector(0, 150);
            mato.Angle = Angle.FromDegrees(90);
        }
        private void KaannyAlas()
        {
            
            mato.Velocity = new Vector(0, -150);
            mato.Angle = Angle.FromDegrees(270);
        }
        private void KaannyVasen()
        {
            
            mato.Velocity = new Vector(-150, 0);
            mato.Angle = Angle.FromDegrees(180);
        }
        private void KaannyOikea()
        {
            
            mato.Velocity = new Vector(150, 0);
            mato.Angle = Angle.FromDegrees(0);
        }

        private void LuoSeina(Vector paikka, double leveys, double korkeus)
            {
                PhysicsObject seina = new PhysicsObject(leveys, korkeus);
                seina.Position = paikka;
                seina.Image = seinakuva;
                seina.Tag = "seina";
                seina.MakeStatic();
                Add(seina);
            }
        

        
        private PhysicsObject LuoMato()
        {
            mato = new LiikkuvaMato(seinaleveys, seinakorkeus);
            mato.Image = Matopeli.matopaakuva;
            mato.Tag = "mato";
            mato.Velocity = new Vector(150, 0);
            AddCollisionHandler(mato, "seina", TormaaSeinaan);
            AddCollisionHandler(mato, "omena", SyoOmenan);
            return mato;

        }

        private PhysicsObject LuoOmena()
        {
            omena = new LiikkuvaMato(seinaleveys, seinakorkeus);
            omena.Image = omenakuva;
            omena.Tag = "omena";
            return omena;
        }
        
        
        private void TormaaSeinaan(IPhysicsObject mato,IPhysicsObject seina)
        {
            mato.Velocity = new Vector(0,0);
            seina.Velocity = new Vector(0, 0);
            
        }

        private void SyoOmenan(IPhysicsObject mato, IPhysicsObject omena)
        {
            omena.Destroy();
        }

    }
    


    public class LiikkuvaMato : PhysicsObject
    {
        public LiikkuvaMato(double leveys, double korkeus)
            : base(leveys, korkeus)
        {
            Mass = 10000;
            IgnoresGravity = true;
            IsUpdated = true;
        }
    }       
        





        




    